@@ -58,15 +58,17 @@ sub new {
     # depending on how $resolver and $rejector are used.
     my $resolver = sub {
         $$value_sr = $_[0];
-        bless $value_sr, _RESOLUTION_CLASS();
 
         # NB: UNIVERSAL::can() is used in order to avoid an eval {}.
         # It is acknowledged that many Perl experts strongly discourage
         # use of this technique.
         if ( UNIVERSAL::can( $$value_sr, 'then' ) ) {
-            _repromise( $value_sr, \@children, $value_sr );
+            return _repromise( $value_sr, \@children, $value_sr );
         }
-        elsif (@children) {
+
+        bless $value_sr, _RESOLUTION_CLASS();
+
+        if (@children) {
             $_->_settle($value_sr) for splice @children;
         }
     };
@@ -171,20 +173,33 @@ sub _repromise {
 #    return (_PENDING_CLASS ne ref $_[0][ _VALUE_SR_IDX ]);
 #}
 
-my ($settle_is_rejection, $self_is_finally);
-
 # This method *only* runs to “settle” a promise.
 sub _settle {
     my ( $self, $final_value_sr ) = @_;
 
     die "$self already settled!" if _PENDING_CLASS ne ref $self->[_VALUE_SR_IDX];
 
-    $self_is_finally = $self->[_IS_FINALLY_IDX];
-
-    $settle_is_rejection = _REJECTION_CLASS eq ref $final_value_sr;
+    my $settle_is_rejection = _REJECTION_CLASS eq ref $final_value_sr;
 
+    # This has to happen up-front or else we can get spurious
+    # unhandled-rejection warnings in asynchronous mode.
     delete $_UNHANDLED_REJECTIONS{$final_value_sr} if $settle_is_rejection;
 
+    if ($Promise::ES6::_EVENT) {
+        _postpone( sub {
+            $self->_settle_now($final_value_sr, $settle_is_rejection);
+        } );
+    }
+    else {
+        $self->_settle_now($final_value_sr, $settle_is_rejection);
+    }
+}
+
+sub _settle_now {
+    my ( $self, $final_value_sr, $settle_is_rejection ) = @_;
+
+    my $self_is_finally = $self->[_IS_FINALLY_IDX];
+
     # A promise that new() created won’t have on-settle callbacks,
     # but a promise that came from then/catch/finally will.
     # It’s a good idea to delete the callbacks in order to trigger garbage
@@ -281,8 +296,7 @@ sub _settle {
 
         return _repromise( @{$self}[ _VALUE_SR_IDX, _CHILDREN_IDX, _VALUE_SR_IDX, _IS_FINALLY_IDX ] );
     }
-
-    if ( @{ $self->[_CHILDREN_IDX] } ) {
+    elsif ( @{ $self->[_CHILDREN_IDX] } ) {
         $_->_settle( $self->[_VALUE_SR_IDX] ) for splice @{ $self->[_CHILDREN_IDX] };
     }
 
@@ -296,10 +310,10 @@ sub DESTROY {
         warn( ( '=' x 70 ) . "\n" . 'XXXXXX - ' . ref( $_[0] ) . " survived until global destruction; memory leak likely!\n" . ( "=" x 70 ) . "\n" );
     }
 
-    if ( my $promise_value_sr = $_[0][_VALUE_SR_IDX] ) {
+    if ( defined $_[0][_VALUE_SR_IDX] ) {
+        my $promise_value_sr = $_[0][_VALUE_SR_IDX];
         if ( my $value_sr = delete $_UNHANDLED_REJECTIONS{$promise_value_sr} ) {
-            my $ref = ref $_[0];
-            warn "$ref: Unhandled rejection: $$value_sr";
+            warn "$_[0]: Unhandled rejection: $$value_sr";
         }
     }
 }
